#! /usr/bin/perl -w
#
# Tool tool generate iconrc file for GTK+-2.2 from a file listing
# the icons and a directory full of icons.
#
# Copyright 2003, Red Hat, Inc.
#
# Permission to use, copy, modify, distribute, and sell this software and its
# documentation for any purpose is hereby granted without fee, provided that
# the above copyright notice appear in all copies and that both that
# copyright notice and this permission notice appear in supporting
# documentation, and that the name of Red Hat not be used in advertising or
# publicity pertaining to distribution of the software without specific,
# written prior permission.  Red Hat makes no representations about the
# suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
#
# RED HAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL RED HAT
# BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
# OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

if (@ARGV != 4) {
    die "Usage: make-iconrc.pl OUTPUT_FILE LIST ICON_DIR INSTALL_ICON_DIR";
}

my $output_file=$ARGV[0];
my $tmp_file="$output_file.temp";
my $list_file = $ARGV[1];
my $icon_dir=$ARGV[2];
my $install_icon_dir=$ARGV[3];

my %icons;

sub read_size_dir {
    my ($dir, $size) = @_;
    
    opendir SDIR, $dir || die "Cannot open $dir: $!\n";

    while (defined ($f = readdir SDIR)) {
	next if $f =~ /^\./;
	next if ! -f "$dir/$f";
	next if $f !~ /.png$/;

	my $id;
	($id = $f) =~ s/.png$//;

	$icons{$id}->{sizes}->{$size}->{have_png} = 1;
    }

    closedir SDIR;
}

sub read_top_dir {
    my ($dir) = @_;

    opendir TDIR, $dir || die "Cannot open $dir: $!\n";

    while (defined ($f = readdir TDIR)) {
	next if $f =~ /^\./;
	next if ! -d "$dir/$f";
	next if ! -d "$dir/$f/stock";
	next if $f !~ /^(\d+)x\1$/;
	
	my $size = $1;
	
	read_size_dir ("$dir/$f/stock", $size);
    }

    closedir TDIR;
};

sub handle_size {
    my ($id,$name,$size) = @_;

    if (!$icons{$id}->{sizes}->{$size}->{have_png}) {
	die "Cannot find PNG file for $id at size $size\n";
    }

    if (exists $icons{$id}->{sizes}->{$size}->{name}) {
	push @{$icons{$id}->{sizes}->{$size}->{name}}, $name;
    } else {
	$icons{$id}->{sizes}->{$size}->{name} = [ $name ];
    }
    
}

read_top_dir($icon_dir);

open LIST, $list_file || die "Cannot open $list_file: $!\n";
while (<LIST>) {
    chomp;
    s/\#.*$//;
    s/\s*?$//;
    s/^\s*//;
    next if $_ eq "";

    if (!/^([A-Za-z][A-Za-z-0-9-]*)\s+(\d+)((\s+[A-Za-z-]+,\d+)*)\s*$/) {
	die "Cannot parse line: $_\nExpected: stock-id default-size size-name,size ...\n";
    }

    my $id = $1;
    my $default_size = $2;
    my $extra_sizes = $3;

    if ($icons{$id} && $icons{$id}->{sizes}->{listed}) {
	die "Duplicate entry for $id\n";
    }
    
    handle_size ($id, "*", $default_size);
    while ($extra_sizes =~ /([A-Za-z-]+),(\d+)/g) {
	handle_size ($id, $1, $2);
    }
}

open ICONRC, ">$tmp_file" || die "Cannot open $tmp_file: $!\n";
END {
    defined $tmp_file && unlink ($tmp_file);
};

print ICONRC <<EOT;
pixmap_path "$install_icon_dir"
style "icon-style"
{
EOT

for $id (sort keys %icons) {
    @sizes = keys %{$icons{$id}->{sizes}};
    @sizes = reverse sort { $a <=> $b } @sizes;

    my @listed;
    my @unlisted;
    for my $size (@sizes) {
	if (!exists $icons{$id}->{sizes}->{$size}->{name}) {
	    push @unlisted, $size;
	} else {
	    push @listed, $size;
	}
    }
#    if (@unlisted) {
#	warn "$id has unlisted sizes ", join (",", @unlisted),"\n";
#    }

    if (@listed == 0) {
	# skip
    } elsif (@listed == 1 && @{$icons{$id}->{sizes}->{$listed[0]}->{name}} == 1) {
	my $size = $listed[0];
	print ICONRC qq(  stock["$id"] = {{ "${size}x${size}/stock/$id.png", *, *, * }}\n);
    } else {
	print ICONRC qq(  stock["$id"] = \{\n);

	my $first = 1;
	for my $size (@listed) {
	    $names = $icons{$id}->{sizes}->{$size}->{name};
	    for $symbolic (@$names) {
		print ICONRC ",\n" unless $first;
		$first = 0;
	    
		print ICONRC qq(    { "${size}x${size}/stock/$id.png", *, *, "$symbolic" });
	    }
	    
	}
	print ICONRC qq(\n  }\n);
    }
    
}

print ICONRC <<EOT;
}
class "GtkWidget" style "icon-style"
EOT

close ICONRC || die "Error writing temporary file: $!\n";
rename $tmp_file, $output_file || die "Error renaming temporary file: $!\n";

